@@ -30,10 +30,13 @@ void RGWOp_User_Info::execute()
   RGWUserAdminOpState op_state;
 
   std::string uid;
+  bool fetch_stats;
 
   RESTArgs::get_string(s, "uid", uid, &uid);
+  RESTArgs::get_bool(s, "stats", false, &fetch_stats);
 
   op_state.set_user_id(uid);
+  op_state.set_fetch_stats(fetch_stats);
 
   http_ret = RGWUserAdminOp_User::info(store, op_state, flusher);
 }
@@ -68,7 +71,7 @@ void RGWOp_User_Create::execute()
   bool exclusive;
 
   uint32_t max_buckets;
-  int32_t key_type = KEY_TYPE_UNDEFINED;
+  uint32_t default_max_buckets = s->cct->_conf->rgw_user_max_buckets;
 
   RGWUserAdminOpState op_state;
 
@@ -81,7 +84,7 @@ void RGWOp_User_Create::execute()
   RESTArgs::get_string(s, "user-caps", caps, &caps);
   RESTArgs::get_bool(s, "generate-key", true, &gen_key);
   RESTArgs::get_bool(s, "suspended", false, &suspended);
-  RESTArgs::get_uint32(s, "max-buckets", RGW_DEFAULT_MAX_BUCKETS, &max_buckets);
+  RESTArgs::get_uint32(s, "max-buckets", default_max_buckets, &max_buckets);
   RESTArgs::get_bool(s, "system", false, &system);
   RESTArgs::get_bool(s, "exclusive", false, &exclusive);
 
@@ -111,6 +114,7 @@ void RGWOp_User_Create::execute()
     op_state.set_secret_key(secret_key);
 
   if (!key_type_str.empty()) {
+    int32_t key_type = KEY_TYPE_UNDEFINED;
     if (key_type_str.compare("swift") == 0)
       key_type = KEY_TYPE_SWIFT;
     else if (key_type_str.compare("s3") == 0)
@@ -119,7 +123,7 @@ void RGWOp_User_Create::execute()
     op_state.set_key_type(key_type);
   }
 
-  if (max_buckets != RGW_DEFAULT_MAX_BUCKETS)
+  if (max_buckets != default_max_buckets)
     op_state.set_max_buckets(max_buckets);
 
   if (s->info.args.exists("suspended"))
@@ -166,7 +170,6 @@ void RGWOp_User_Modify::execute()
   bool system;
 
   uint32_t max_buckets;
-  int32_t key_type = KEY_TYPE_UNDEFINED;
 
   RGWUserAdminOpState op_state;
 
@@ -214,6 +217,7 @@ void RGWOp_User_Modify::execute()
     op_state.set_generate_key();
 
   if (!key_type_str.empty()) {
+    int32_t key_type = KEY_TYPE_UNDEFINED;
     if (key_type_str.compare("swift") == 0)
       key_type = KEY_TYPE_SWIFT;
     else if (key_type_str.compare("s3") == 0)
@@ -459,7 +463,6 @@ void RGWOp_Key_Create::execute()
   std::string secret_key;
   std::string key_type_str;
 
-  int32_t key_type = KEY_TYPE_UNDEFINED;
   bool gen_key;
 
   RGWUserAdminOpState op_state;
@@ -488,6 +491,7 @@ void RGWOp_Key_Create::execute()
     op_state.set_generate_key();
 
   if (!key_type_str.empty()) {
+    int32_t key_type = KEY_TYPE_UNDEFINED;
     if (key_type_str.compare("swift") == 0)
       key_type = KEY_TYPE_SWIFT;
     else if (key_type_str.compare("s3") == 0)
@@ -520,8 +524,6 @@ void RGWOp_Key_Remove::execute()
   std::string access_key;
   std::string key_type_str;
 
-  int32_t key_type = KEY_TYPE_UNDEFINED;
-
   RGWUserAdminOpState op_state;
 
   RESTArgs::get_string(s, "uid", uid, &uid);
@@ -540,6 +542,7 @@ void RGWOp_Key_Remove::execute()
     op_state.set_access_key(access_key);
 
   if (!key_type_str.empty()) {
+    int32_t key_type = KEY_TYPE_UNDEFINED;
     if (key_type_str.compare("swift") == 0)
       key_type = KEY_TYPE_SWIFT;
     else if (key_type_str.compare("s3") == 0)
@@ -691,6 +694,7 @@ void RGWOp_Quota_Info::execute()
   if (http_ret < 0)
     return;
 
+  flusher.start(0);
   if (show_all) {
     UserQuotas quotas(info);
     encode_json("quota", quotas, s->formatter);
